#coding: utf-8

import json

from itertools import groupby


class ContextStore(object):
    def __init__(self, rdb, max_records=10):
        self.rdb = rdb
        # Ограничивать количество записей в каждом множестве, хранимом в Redis
        self.max_records = max_records

    def get_identity(self, d):
        '''
        Получаем идентификатор, который станет определяющей частью в
        имени множества.
        '''
        parts = [d['region'], d['client'], d['product']]
        if 'instance' in d:
            parts.append(d['instance'])
        return '.'.join(parts)


    def put(self, identity, name, ctx, timestamp):
        '''
        Добавляем часть контекста в множество. Если элемент уже есть в 
        множестве, то его копия не добавляется.
        '''
        
        set_name = '#'.join(['context', self.get_identity(identity), name])
        el = self.rdb.lindex(set_name, 0)
        if el:
            d = eval(el)
            if d['info'] == ctx:
                return False

        self.rdb.lpush(set_name, dict(info=ctx, timestamp=timestamp))
        self.rdb.ltrim(set_name, 0, self.max_records)
        return True
 

    def get(self, identity, names, start, end):
        '''
        Получаем все части контекстов указанной системы за определенный
        период, и сливаем их в сгруппированные по времени структуры.
        '''
        identity_str = self.get_identity(identity)
        parts = []
        contexts = []

        for ctx_part_name in names:
            set_name = '#'.join(['context', identity_str, ctx_part_name])
            elements = self.rdb.lrange(set_name, 0, self.max_records)

            import pdb
            pdb.set_trace()
            elements = map(eval, elements)
            elements = map(lambda x: (x['info'], x['timestamp'], ctx_part_name), elements)
            elements = filter(lambda x: start < x[1] <= end, elements)            

        # Группируем части контекстов по временным меткам. Предполагается,
        # что в базу принадлежащие одному и тому же контексту части
        # заносились одновременно.
        for key, group in groupby(sorted(elements, key=lambda x: x[1]), lambda x: x[1]):
            result = {
                'info': {},
                'timestamp': key
            }
            for part in group:
                result['info'][part[2]] = part[0]
            contexts.append(result)
        return contexts
